#include "program_version.h"
#include <hardware/btn-hardware.h> // for btn_hardware_init
#include <led/btn-led.h> // for btn_led_init
#include <sys/timer.h> // for NutSleep
#include <stdio.h>
#include <wsnlab/wsnlab.h>
#include <arch/timer.h>
#include <terminal/btn-terminal.h>
#include <terminal/btn-cmds.h>
#include <terminal/nut-cmds.h>
#include <string.h>
#include <sys/thread.h>
#include <sys/event.h>

#define PROJECT_NAME "t1-example"
#define UART_SPEED   57600

HANDLE notifyAlice;
HANDLE notifyBob;
HANDLE timer;

/**
 * Initializes the BTnode.
 */
static void btnode_init(void) 
{    
    /* initialize the btnode hardware */
    btn_hardware_init();
    btn_led_init(0);

    char prompt[TERMINAL_MAX_PROMPT_LENGTH];
    u_short mac_address = get_MAC_address_from_EEPROM();

    sprintf( prompt, "[%s@%02X:%02X]$", PROJECT_NAME, mac_address >> 8, 
    	mac_address & 0xFF );
    
    term_init(UART_SPEED, prompt); 
}

/**
 * Registers all supported commands.
 */
static void register_cmds(void)
{
	nut_cmds_register_cmds();
    btn_cmds_register_cmds();
    wsnlab_cmds_register_cmds();
}


THREAD(ALICE,e)
{
	for(;;)
	{
		NutEventWait(&notifyAlice, NUT_WAIT_INFINITE);
		printf("Alice active\n");
		NutSleep(5000);
		NutEventPost(&notifyBob);
	}
}


THREAD(BOB,e)
{
	for(;;)
	{
		NutEventWait(&notifyBob, NUT_WAIT_INFINITE);
		printf("Bob active\n");
		NutSleep(5000);
		NutEventPost(&notifyAlice);
	}
}

static void tm_callback(HANDLE h, void* p)
{
	printf( "Timer active\n" );
}

int main(void)
{
	btnode_init();
	
	DEBUGB("\n# ----------------------------------------");
    DEBUGB("\n# " PROJECT_NAME " version " PROGRAM_VERSION "\n");
    DEBUGB("# ----------------------------------------\n");
    NutSleep(200);
	
	register_cmds();

	// create two threads
	NutThreadCreate( "Alice", ALICE, NULL, 196 );
	NutThreadCreate( "Bob", BOB, NULL, 196 );
	
	// create a timer
	timer = NutTimerStart(3000, tm_callback, NULL, 0);
	
	// activate alice
	NutEventPost(&notifyAlice);
	
	btn_terminal_run(BTN_TERMINAL_NOFORK, 0);

	for(;;)	NutSleep(100);

	// main should never return (where to?!)
	return 1;
}
